#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "progressdialog.h"
#include "welcome.h"

#include <QMessageBox>
#include <QtNetwork/QFtp>
#include <QtNetwork/QHttp>
#include <QFile>
#include <QStringList>
#include <QInputDialog>
#include <QDir>
#include <QtXml/QtXml>
#include <QSystemTrayIcon>
#include <QMenu>
#include <QAction>
#include <QTimer>
#include <QDate>
#include <QTime>
#include <QString>

#define TESTFTP 1
#define DOWNLOAD 2

#define VERSION "0.95"

//Windows-Compatibility
#ifdef _WIN32
    #define SLASH "\\"
#else
    #define SLASH "/"
#endif

//#define PATH_CONFIG QDir::homePath() + SLASH + ".config" + SLASH + "FTPackup"
#define PATH_LAST "last"
#define PATH_PSTATE "pstate"
//#define PATH_SETTINGS "settings"
#define PATH_FILELIST "files"
#define PATH_DEFSAVEDIR QDir::homePath() + SLASH + "FTPackup"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    //Init product info
        QCoreApplication::setOrganizationName("C_Classic");
        QCoreApplication::setOrganizationDomain("CClassicVideos@aol.com");
        QCoreApplication::setApplicationName("FTPackup");

    // foobar
    isLoaded=false;
    restoreAction = new QAction(tr("&Restore"), this);
    connect(restoreAction, SIGNAL(triggered()), this, SLOT(showNormal()));

    quitAction = new QAction(tr("&Quit"), this);
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));

    trayIconMenu = new QMenu(this);
    trayIconMenu->addAction(restoreAction);
    trayIconMenu->addSeparator();
    trayIconMenu->addAction(quitAction);

    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setContextMenu(trayIconMenu);

    trayIcon->setIcon(QIcon(QString::fromUtf8(QString(":/src/icon.png").toStdString().data())));

    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
            this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));

    trayIcon->show();

    LoadSettings();

    tim = new QTimer;
    tim->setInterval(60000);
    connect(tim, SIGNAL(timeout()), this, SLOT(timerovf()));
    if (ui->chkEnabled->checkState()) {
        tim->start(); //If sheduler is enabled start timer
    }
    isLoaded = true;
    //timerovf();

    QSettings settings;
    if (settings.value("welcome", "0").toString() != "1") {
        whatsNew();
    }
}

void MainWindow::AboutIcons() {
    QMessageBox::information(this, tr("About icons..."), "Iconset \"kAwOken\" made by \"alecive87@gmail.com\"\nhttp://alecive.deviantart.com/ \n\n"
                             "Iconset \"gnome-Colors\" made by \"perfectska04@gmail.com\" and two other people.\n"
                             "http://code.google.com/p/gnome-colors/");
}

void MainWindow::whatsNew() {
    //if (welcomescreen) return;
    // Got to fix this somewhen else
    welcomescreen = new welcome(this);
    if (welcomescreen->isVisible()) {
        welcomescreen->setFocus();
    }
    else {
        welcomescreen->show();
    }
}

void MainWindow::RewriteTS() {
    //Save current date and time to the file
    QSettings settings;
    settings.setValue(PATH_LAST, QDate::currentDate().toString("dd.MM.yyyy") + QTime::currentTime().toString(";hh:mm"));
}

void MainWindow::EnabledChanged() {
    if (!ui->chkEnabled->checkState()) ui->lblTimeNextBackup->setText(tr("Next backup in %1 min.").arg("---"));
    else timerovf();
    ui->numMode->setEnabled(ui->chkEnabled->isChecked());
    ui->comboMode->setEnabled(ui->chkEnabled->isChecked());
    ui->lblTimeNextBackup->setEnabled(ui->chkEnabled->isChecked());
    ui->lblMode->setEnabled(ui->chkEnabled->isChecked());
}

void MainWindow::timerovf() {
    if (!isLoaded || !this->isEnabled()) return;

    // Get when last backup was made
     QSettings settings;
     QString last = settings.value(PATH_LAST,"").toString();
     if (last != "") {
         QStringList DateVal;
         DateVal = last.replace(10,"").split(";"); //0:date;1:time
         int lastbackup = QTime::fromString(DateVal.at(1),"hh:mm").secsTo(QTime::currentTime()) + QDate::fromString(DateVal.at(0),"dd.MM.yyyy").daysTo(QDate::currentDate())*24*60*60;
         int diff;
         //QMessageBox::information(this,"tim",QString::number(lastbackup));
         // Minutes
         // Hours
         // Days
         // Weeks
         switch (ui->comboMode->currentIndex()) {
         case 0:
             diff = 60; //60s=1m
             break;
         case 1:
             diff = 60*60;//1h
             break;
         case 2:
             diff = 60*60*24;//1d
             break;
         case 3:
             diff = 60*60*24*7;//1w
             break;
         }
         diff *= ui->numMode->value();

         //Refresh Label
         int days, hours, minutes;
         minutes = ((diff-lastbackup)/60 + 1);
         hours = ((double) minutes/(60));
         days = (double) hours/24;

         minutes = minutes%60;
         hours = hours%24;

         ui->lblTimeNextBackup->setText(tr("Next backup in %1 day%4, %2 hour%5 and %3 minute%6").arg(
                                            QString::number(days), QString::number(hours),QString::number(minutes),
                                            (days != 1 ? tr("s"):""),
                                            (hours != 1 ? tr("s"):""),
                                            (minutes != 1 ? tr("s"):"")));

         if (!(lastbackup> diff)) return; //No update --> Quit function!
     }
     else return; //file did not exist
     //Stop timer (only one update a time)
     tim->stop();

     //Update if time is ready or no update-file exists.
     if (ui->listFiles->count() < 1 && !(ui->chkFTPBackup->checkState() || ui->chkLocalCopy->checkState())) return;
     prg_visible = ui->radioVisible->isChecked();
     prg_autohide = ui->radioVisible->isChecked();
     startBackup();
     // RewriteTS(); Is now done automatically
}

void MainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason) {
    switch (reason) {
    case QSystemTrayIcon::Trigger:
    case QSystemTrayIcon::DoubleClick:
        this->setVisible(!this->isVisible());
        break;
    case QSystemTrayIcon::MiddleClick:
       // showMessage();
        break;
    default:
        ;
    }
}

MainWindow::~MainWindow()
{
    SaveSettings();
    delete ui;
}

void MainWindow::LoadSettings() {
    //Load settings
    QSettings settings;
    //Load files
    ui->listFiles->clear();
    {
        QString pstate = settings.value(PATH_PSTATE,"").toString();
        if (pstate != "") {
            {
            if (pstate != "0") {
                #ifndef _WIN32
                    QMessageBox::warning(this,tr("Program crashed!"),tr("WARNING: The program wasn't closed correctly the last time!"));
                #endif
            }
            settings.setValue(PATH_PSTATE, "1");
            }

           // Defaults
                ui->txtTempDir->setText(QDir::tempPath());
                ui->txtSaveDir->setText(PATH_DEFSAVEDIR);
                ui->chkShowErrors->setChecked(1);

            //Text and numbers
               ui->txtServer->setText(settings.value("texts/server", ui->txtServer->text()).toString());
               ui->numPort->setValue(settings.value("texts/port", ui->numPort->value()).toInt());
               ui->txtUsername->setText(settings.value("texts/username", ui->txtUsername->text()).toString());
               ui->txtPassword->setText(settings.value("texts/password", ui->txtPassword->text()).toString());
               ui->txtTempDir->setText(settings.value("texts/tempDir", ui->txtTempDir->text()).toString());
               ui->txtSaveDir->setText(settings.value("texts/saveDir", ui->txtSaveDir->text()).toString());
               ui->txtFTPBackupPath->setText(settings.value("texts/ftpBackupPath", ui->txtFTPBackupPath->text()).toString());
               ui->numMode->setValue(settings.value("texts/mode", ui->numMode->value()).toInt());
               ui->txtDirFormatDate->setText(settings.value("texts/dirFormatDate", ui->txtDirFormatDate->text()).toString());
               ui->txtDirFormatTime->setText(settings.value("texts/dirFormatTime", ui->txtDirFormatTime->text()).toString());

            //Checkboxes
               ui->chkEnabled->setChecked(settings.value("checkbox/enabled", ui->chkEnabled->checkState()).toBool());
               ui->chkLocalCopy->setChecked(settings.value("checkbox/localCopy", ui->chkLocalCopy->checkState()).toBool());
               ui->chkFTPBackup->setChecked(settings.value("checkbox/ftpBackup", ui->chkFTPBackup->checkState()).toBool());
               ui->radioVisible->setChecked(settings.value("checkbox/visible", ui->radioVisible->isChecked()).toBool());
               ui->chkShowErrors->setChecked(settings.value("checkbox/showErrors", ui->chkShowErrors->checkState()).toBool());


            // Combo
               ui->comboMode->setCurrentIndex(settings.value("combo/mode", ui->comboMode->currentIndex()).toInt());

       /* try {

            QString setting = settings.value(PATH_SETTINGS).toString();
            QStringList settingslist = QString(setting.replace(QString(10),"")).split(";"); //Read all, remove all LF (newline) and split on ;
            if (settingslist.count() != 16) throw 1;
            ui->txtServer->setText(settingslist.at(0));
            ui->numPort->setValue(QString(settingslist.at(1)).toInt());
            ui->txtUsername->setText(settingslist.at(2));
            ui->txtPassword->setText(settingslist.at(3));
            ui->txtTempDir->setText(settingslist.at(4));
            ui->txtSaveDir->setText(settingslist.at(5));
            ui->txtFTPBackupPath->setText(settingslist.at(6));
            ui->chkEnabled->setChecked( QString(settingslist.at(7)).toInt());
            ui->chkLocalCopy->setChecked( QString(settingslist.at(8)).toInt());
            ui->chkFTPBackup->setChecked( QString(settingslist.at(9)).toInt());
            ui->numMode->setValue(QString(settingslist.at(10)).toInt());
            ui->comboMode->setCurrentIndex(QString(settingslist.at(11)).toInt());
            ui->radioVisible->setChecked(QString(settingslist.at(12)).toInt());
            ui->radioBackground->setChecked(!QString(settingslist.at(12)).toInt());
            ui->txtDirFormatDate->setText(settingslist.at(13));
            ui->txtDirFormatTime->setText(settingslist.at(14));
            ui->chkShowErrors->setChecked( QString(settingslist.at(15)).toInt());
            }
            catch (int e) {
                QMessageBox::critical(this,"ERROR",QString(tr("Error loading settings! (%1)<br><i>Will save default settings when quitting.</i>")).arg(QString::number(e)));
            }
            }*/

        } // pstate was drin
        else {
            //Config for first start (Will be saved when quitting)
            ui->txtTempDir->setText(QDir::tempPath());
            ui->txtSaveDir->setText(PATH_DEFSAVEDIR);
            ui->chkShowErrors->setChecked(1);
            QMessageBox::information(this,tr("FTPackup - first start"),tr("The program has loaded the default values!"));
        }
    }

    reloadFiles();
}

void MainWindow::reloadFiles() {
    {
    QSettings settings;
    QString fl = settings.value(PATH_FILELIST, "").toString();
    if (fl != "") {
        if (fl != "") {
            QStringList files = QString(fl.replace(QString(10),"")).split(";"); //Read all, remove all LF (newline) and split on ;
            for (int x=0; x < files.count(); x++) {
                ui->listFiles->addItem(files.at(x));
            }
        }
    }
    }
}

void MainWindow::SaveSettings() {
    QSettings settings;

    if (settings.value(PATH_LAST, "").toString() == "") { // we assume the program wasn't started before
        RewriteTS(); //and a dummy backup (Timestamp now)
    }
    //Save File-list
        {
        QString temp="";
        for (int x=0; x < ui->listFiles->count();x++) {
            temp += ui->listFiles->item(x)->text();
            if (x < ui->listFiles->count()-1) temp += ";";
        }
        settings.setValue(PATH_FILELIST, temp);
        }


    //Save other settings
        {

        //Text and numbers
        settings.setValue("texts/server", ui->txtServer->text());
        settings.setValue("texts/port", ui->numPort->value());
        settings.setValue("texts/username", ui->txtUsername->text());
        settings.setValue("texts/password", ui->txtPassword->text());
        settings.setValue("texts/tempDir", ui->txtTempDir->text());
        settings.setValue("texts/saveDir", ui->txtSaveDir->text());
        settings.setValue("texts/ftpBackupPath", ui->txtFTPBackupPath->text());
        settings.setValue("texts/mode", ui->numMode->value());
        settings.setValue("texts/dirFormatDate", ui->txtDirFormatDate->text());
        settings.setValue("texts/dirFormatTime", ui->txtDirFormatTime->text());

        //Checkboxes
        settings.setValue("checkbox/enabled", ui->chkEnabled->checkState());
        settings.setValue("checkbox/localCopy", ui->chkLocalCopy->checkState());
        settings.setValue("checkbox/ftpBackup", ui->chkFTPBackup->checkState());
        settings.setValue("checkbox/visible", ui->radioVisible->isChecked());
        settings.setValue("checkbox/showErrors", ui->chkShowErrors->checkState());


        // Combo
        settings.setValue("combo/mode", ui->comboMode->currentIndex());
        }


    //Set program-state to 0
        {
        settings.setValue(PATH_PSTATE, "0");
        }
}

void MainWindow::AboutQt() {
    QMessageBox::aboutQt(this);
}

void MainWindow::ShowError(QString title, QString text) {
    ui->listErrors->addItem(QDate::currentDate().toString("dd.MM.yyyy ") + QTime::currentTime().toString("hh:mm:ss") + " - " + title + ": " + text);
    if (ui->chkShowErrors->isChecked() || ShowErrs) {
        QMessageBox::critical(this,title,text);
    }
}

int MainWindow::DownloadFile(QString src, QString dest) {
        file = new QFile(dest);
        if (!file->open(QIODevice::WriteOnly)) {
            ShowError(tr("FTP"), tr("Unable to save the file %1: %2.").arg(dest).arg(file->errorString()));
            this->setEnabled(true);
            delete file;
            return 1;
        }
        ftp->get(src, file);
        return 0;
}

//DOESN'T WORK
int MainWindow::UploadFile(QString src, QString dest) {
        file = new QFile(src);
        if (!file->open(QIODevice::ReadWrite)) {
            QMessageBox::information(this, tr("FTP"),
                                     tr("Unable to read the file %1: %2.")
                                     .arg(src).arg(file->errorString()));
            delete file;
            return 1;
        }
        ftp->put(file, "foo");
        return 0;
}



void MainWindow::TestConnection() {
    ShowErrs=true;
    FTP_Command = TESTFTP;
    FTP_Connect();
}

void MainWindow::BackupNow() {
    if (ui->listFiles->count() < 1 || !(ui->chkFTPBackup->checkState() || ui->chkLocalCopy->checkState())) return;
    prg_visible = true;
    prg_autohide = false;
    startBackup();
}

void MainWindow::startBackup() {
    this->setEnabled(false);
    Timestamp = QDate::currentDate().toString(ui->txtDirFormatDate->text()) + QTime::currentTime().toString(ui->txtDirFormatTime->text());
    FTP_Command = DOWNLOAD;
    FTP_Connect();
}

void MainWindow::FTP_Connect() {
    ftp = new QFtp(this); //Create a new ftp-instance

    connect(ftp, SIGNAL(commandFinished(int,bool)), //We need to get noticed when commands are finished.
            this, SLOT(cmdFinished(int,bool)));
    connect(ftp, SIGNAL(dataTransferProgress(qint64,qint64)),
            this, SLOT(updateDataTransferProgress(qint64,qint64)));

    ftp->connectToHost(ui->txtServer->text(),ui->numPort->value()); //Connect to the specified server.
}

void MainWindow::FTP_Disconnect() {
    ftp->close();
    ftp->abort();
    ftp->deleteLater();
    ftp = 0;
}


void MainWindow::cmdFinished(int, bool error) {
    //QMessageBox::information(this,"OK","cmdFinished");

    if (ftp->currentCommand() == QFtp::ConnectToHost) { //When connected  log in
        if (error) {
            ShowError(tr("FTP"),
                                     tr("Unable to connect to the FTP server "
                                        "at %1. Please check that the host "
                                        "name is correct.")
                                     .arg(ui->txtServer->text()));
            FTP_Disconnect();
            this->setEnabled(true);
            return;
        }

        if (ui->txtUsername->text() != "") {
            ftp->login(ui->txtUsername->text(),ui->txtPassword->text());
        }
        else ftp->login(); //anonymous


    }

    else if (ftp->currentCommand() == QFtp::LoggedIn) { //When logged in
        if (error) {
            ShowError(tr("FTP"),
                                     tr("Unable to connect to the FTP server "
                                        "at %1. Please check that the username "
                                        "and password is correct.")
                                     .arg(ui->txtServer->text()));
            FTP_Disconnect();
            this->setEnabled(true);
            return;
        }

        //Connection Okay
        // Jetzt prüfen, was gemacht werden soll
        if (FTP_Command == TESTFTP) {
            QMessageBox::information(this,tr("Test connection"),tr("Connection OK"));
            ShowErrs=false;
            FTP_Disconnect();
        }
        else if (FTP_Command == DOWNLOAD) {
            if (prg_visible) prgdlg=new progressDialog(this);
            if (prg_visible) prgdlg->show();
            fid=0;
            NextFile();
        }
        return;
    }

    else if (ftp->currentCommand() == QFtp::Get) {
        if (error) {
            //QMessageBox::critical(this,"ERROR DOWNLOADING FILE!","^");
            if (prg_visible) prgdlg->AddLog("ERROR DOWNLOADING FILE " + QString::number(fid) + " (" + ui->listFiles->item(fid-1)->text() + ")");
            file->close();
            file->remove();
        } else {
            //QMessageBox::information(this,"OKAY","FILE OK");


            QString filename1=file->fileName();
            //Close file.
            file->close();

            //Copy file to destination
            //  Check whether dir exists - if not create it
            QString temp = ui->listFiles->item(fid-1)->text().replace("\\","/");
            QStringList temparr = temp.split("/");
            temparr.removeAt(temparr.count()-1); //Path will remain
            QString dir = temparr.join(SLASH);
            QDir foo(ui->txtSaveDir->text() + SLASH + Timestamp + SLASH + dir);
            if (!foo.exists()) foo.mkpath(ui->txtSaveDir->text() + SLASH + Timestamp + SLASH + dir + SLASH);
            //  Copy it! But only if we want a local copy
            if (ui->chkLocalCopy->checkState()) QFile::copy(filename1, ui->txtSaveDir->text() + SLASH + Timestamp + SLASH + ui->listFiles->item(fid-1)->text());
            // Add Message to the log
            //if (prg_visible) prgdlg->AddLog("File " + QString(temp) + " copied!");

            if (prg_visible) prgdlg->AddLog(tr("File %1 (%2) downloaded!").arg(QString::number(fid)).arg(QString(temp)));
        }
        delete file;
        if (fid < ui->listFiles->count()) NextFile();
        else {
            //prgdlg->close();
            //prgdlg=0;
            //delete prgdlg;
            FTP_Disconnect();
            RewriteTS(); //Rewrite Timestamp
            this->setEnabled(true);
            if (prg_autohide) prgdlg->close();
            timerovf(); //Update label etc.
            if (ui->chkEnabled->checkState()) tim->start(); //If sheduler is enabled restart timer
        }
    }
    else if(ftp->currentCommand() == QFtp::Put) {
        if (error) {
            QMessageBox::information(this,"E","ERROR UPLOADING FILE");
        }
        else {
            QMessageBox::information(this,"OKAY","FILE OK");
        }
    }
}

void MainWindow::NextFile() {
    QString temp = ui->listFiles->item(fid)->text().replace("\\","/");
    QStringList temparr = temp.split("/");

    temparr.removeAt(temparr.count()-1); //Path will remain
    QString dir = temparr.join(SLASH);

    QDir foo(ui->txtTempDir->text() + SLASH + dir);

    if (!foo.exists()) foo.mkdir(ui->txtTempDir->text() + SLASH + dir);
    //QMessageBox::information(this,"-",temp);
    DownloadFile(temp,ui->txtTempDir->text() + SLASH + temp);
    if (prg_visible) prgdlg->SetTotal(ui->listFiles->count(),fid);
    fid++;
}

void MainWindow::AddFile() {
    QString temp = QInputDialog::getText(this,tr("Add new File"),tr("Enter the filename with the full path.<br> index.php;subDir/file.txt"));
    if (temp != "") {
        QStringList temparr = temp.split(";");
        for (int x=0; x < temparr.count();x++) {
                ui->listFiles->addItem(temparr.at(x));
        }
    }
}

void MainWindow::RemoveFile() {
    try {
        for (int x=ui->listFiles->count()-1; x >= 0;x--) {
            if (ui->listFiles->item(x)->isSelected()) {
                ui->listFiles->takeItem(x);
            }
        }
    }
    catch(char const* e) {
        QMessageBox::critical(this,tr("Error"),
                              tr("<h1>Error removing the selected entries!</h1><p>Error %1</p>").arg("foo")
                              );
    }
}

void MainWindow::CheckForUpdates() {
    this->setEnabled(false);
    http = new QHttp("cclassic.users.sourceforge.net"); //cclassic.users.sourceforge.net

    /*connect(http, SIGNAL(done(bool)),
            this, SLOT(httpFinished()));*/
    connect(http, SIGNAL(requestFinished(int,bool)),
            this, SLOT(httpFinished()));

    file = new QFile(QDir::tempPath() + SLASH + "FTPackup_update");
    if (!file->open(QIODevice::WriteOnly)) {
        QMessageBox::information(this, tr("FTP"),
                                 tr("Unable to save the file %1: %2.")
                                 .arg(file->fileName()).arg(file->errorString()));
        delete file;
        return;
    }
    http->get("/ftpackup/cver.txt", file);


}
void MainWindow::httpFinished() {
    file->close();
    //delete file;

    if (file->fileName() == QDir::tempPath() + SLASH + "FTPackup_update") { //Check for updates
        //file = new QFile(QDir::tempPath() + SLASH + "FTPackup_update");
        file->open(QIODevice::ReadOnly);
        upd =QString(file->readAll()).split(";");
        file->close();
        delete file;

        this->setEnabled(true);

        if (upd.at(0) != "FTPACKUP_UPDATE" || upd.count() < 3) {
            QMessageBox::critical(this,tr("ERROR") + "(cclassic.users.sourceforge.net)",QString(tr("Error: File invalid!")) + QString(10) +  QString(tr("If you the error persists, please contact the webmaster at cclassic@users.sourceforge.net")));
        }
        else {
            if (QString(upd.at(1)).toFloat() > QString(VERSION).toFloat()) {
                if (QMessageBox::question(this,tr("Update"),tr("Version %1 is now available!\n\nDo you want to update now?").arg(upd.at(1)),QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes) {
                    /*cUpdate=0;
                    prgdlg=new progressDialog(this);
                    prgdlg->show();
                    prgdlg->SetTotal(0,upd.count() - 3);
                    prgdlg->SetCurrent(0,0);
                    NextUpdateFile();*/
                    QMessageBox::critical(this, tr("Programmer too lazy!"), tr("Sorry - but the updater is not finished already.\nMaybe it will be implemented in a future version of FTPackup.\nPlease download the latest version from the project's website."));
                    prgdlg->close();
                    delete prgdlg;
                }
            }
            else {
                QMessageBox::information(this,tr("Update"),tr("Your version of FTPackup is current!"));
            }
        }
    }
    else {
        //Close file (Just download)
        delete file;
        NextUpdateFile();
    }
}

void MainWindow::NextUpdateFile() {
    cUpdate++;
    prgdlg->SetTotal(cUpdate,upd.count()-4);
    if (cUpdate < upd.count()-3) {
        prgdlg->AddLog(tr("Downloading file %1: %2").arg(QString::number(cUpdate),upd.at(cUpdate + 2)));
        file = new QFile(QDir::currentPath() + SLASH + upd.at(cUpdate + 2));
        if (!file->open(QIODevice::WriteOnly)) {
            QMessageBox::information(this, tr("FTP"),
                                     tr("Unable to save the file %1: %2.")
                                     .arg(file->fileName()).arg(file->errorString()));
            delete file;
            return;
        }
        http->get("/ftpackup/files/" + upd.at(cUpdate + 2),file);
    }
    else {
        //Update finished!
        prgdlg->close();
        delete prgdlg;
        QMessageBox::information(this,tr("Update completed!"),"Click on OK to restart the application.");

    }
}

void MainWindow::ResetSettings() {
    SaveSettings();
    QSettings settings;
    settings.remove("*");
    LoadSettings();
}

void MainWindow::updateDataTransferProgress(qint64 readBytes,
                                           qint64 totalBytes)
{
    if (prg_visible) prgdlg->SetCurrent(readBytes,totalBytes);
}

void MainWindow::CheckLocalChange(int state) {
    ui->txtSaveDir->setEnabled(state);
}

void MainWindow::importFiles() {
    QString pathTo = QInputDialog::getText(this,tr("Import file-list"),tr("Enter path and filename of the source file:<br>e.g.: /home/user/file.txt"),QLineEdit::Normal,QDir::homePath() + SLASH + "file.txt");
    if (pathTo == "") return;
    QFile::copy(pathTo,PATH_FILELIST);
    QMessageBox::information(this,tr("Import file-list"),tr("File imported successfully"));
    reloadFiles();
}

void MainWindow::exportFiles() {
    QString pathTo = QInputDialog::getText(this,tr("Export file-list"),tr("Enter path and filename of the destination file:<br>e.g.: /home/user/file.txt"),QLineEdit::Normal,QDir::homePath() + SLASH + "file.txt");
    if (pathTo == "") return;
    SaveSettings();
    QFile::copy(PATH_FILELIST,pathTo);
    QMessageBox::information(this,tr("Export file-list"),tr("File exported successfully"));
}
